note
	description: "inheritance application root class"
	date: "$Date$"
	revision: "$Revision$"

class
	TEST_INHERITANCE

inherit
	ES_TEST

create
	make

feature -- Initialization

	make
			-- Run tests.
		do
			add_boolean_case (agent test_students)
			add_boolean_case (agent test_dynamic_binding_students)
			add_boolean_case (agent test_type_cast)
			add_boolean_case (agent test_sms_polymorphism)
			add_boolean_case (agent test_sms_dynamic_binding)
			add_violation_case (agent test_smart_phone_type_cast_violation)
			show_browser
			run_espec
		end

feature -- Tests
	test_students: BOOLEAN
		local
			c1, c2: COURSE
			jim: RESIDENT_STUDENT
			jeremy: NON_RESIDENT_STUDENT
		do
			comment ("test_students")
			create c1.make ("EECS2030", 500.0)
			create c2.make ("EECS3311", 500.0)

			create jim.make ("J. Davis")
			jim.set_pr (1.25)
			jim.register (c1)
			jim.register (c2)
			Result := jim.tuition = 1250
			check Result end

			create jeremy.make ("J. Gibbons")
			jeremy.set_dr (0.75)
			jeremy.register (c1)
			jeremy.register (c2)
			Result := jeremy.tuition = 750
		end

	test_students_2: BOOLEAN
		local
			s1, s2, s3: STUDENT
			rs: RESIDENT_STUDENT
			nrs: NON_RESIDENT_STUDENT
			s: STRING
			cs: LINKED_LIST[COURSE]
			c: COURSE
			r, t: REAL
		do
			create c.make ("EECS3311", 500)
			create {STUDENT} s1.make ("S1")
			create {RESIDENT_STUDENT} s2.make ("S2")
			create {RESIDENT_STUDENT} s3.make ("S3")
			create {RESIDENT_STUDENT} rs.make ("RS")
			create {NON_RESIDENT_STUDENT} nrs.make ("NRS")

			-- calling common features declared in STUDENT
			s := s1.name
			s := s2.name
			s := s3.name
			s := rs.name
			s := nrs.name
			cs := s1.courses
			cs := s2.courses
			cs := s3.courses
			cs := rs.courses
			cs := nrs.courses
			s1.register (c)
			s2.register (c)
			s3.register (c)
			rs.register (c)
			nrs.register (c)
			t := s1.tuition
			t := s2.tuition
			t := s3.tuition
			t := rs.tuition
			t := nrs.tuition

			-- calling features specific to RESIDENT_STUDENT
--			s1.set_pr (1.25)
--			s2.set_pr (1.25)
--			s3.set_pr (1.25)
			rs.set_pr (1.25)
--			nrs.set_pr (1.25)
--			r := s1.premium_rate
--			r := s2.premium_rate
--			r := s3.premium_rate
			r := rs.premium_rate
--			r := nrs.premium_rate

			-- calling features specific to NON_RESIDENT_STUDENT
--			s1.set_dr (1.25)
--			s2.set_dr (1.25)
--			s3.set_dr (1.25)
--			rs.set_dr (1.25)
			nrs.set_dr (1.25)
--			r := s1.discount_rate
--			r := s2.discount_rate
--			r := s3.discount_rate
--			r := rs.discount_rate
			r := nrs.discount_rate
		end

	test_polymorphism_students
		local
			jim: STUDENT
			rs: RESIDENT_STUDENT
			nrs: NON_RESIDENT_STUDENT
		do
			create {STUDENT} jim.make ("J. Davis")
			create {RESIDENT_STUDENT} rs.make ("J. Davis")
			create {NON_RESIDENT_STUDENT} nrs.make ("J. Davis")
			jim := rs
--			rs := jim
			jim := nrs
--			rs := jim
		end

	test_dynamic_binding_students: BOOLEAN
		local
			jim: STUDENT
			rs: RESIDENT_STUDENT
			nrs: NON_RESIDENT_STUDENT
			c: COURSE
		do
			comment ("test_dynamic_binding_students")
			create c.make ("EECS3311", 500.0)
			create {STUDENT} jim.make ("J. Davis")

			create {RESIDENT_STUDENT} rs.make ("J. Davis")
			rs.register (c)
			rs.set_pr (1.5)
			jim := rs
			Result := jim.tuition = 750.0
			check Result end

			create {NON_RESIDENT_STUDENT} nrs.make ("J. Davis")
			nrs.register (c)
			nrs.set_dr (0.5)
			jim := nrs
			Result := jim.tuition = 250.0
		end

	test_type_cast: BOOLEAN
		local
			jim: STUDENT
			rs: RESIDENT_STUDENT
			c: COURSE
		do
			comment ("test_type_cast")
			create c.make ("EECS3311", 500.0)
			create {RESIDENT_STUDENT} jim.make ("J. Davis")
--			rs := jim -- won't compile
			check attached {RESIDENT_STUDENT} jim as rs_jim then
				rs := rs_jim
			end
			rs.set_pr (1.5)
			rs.register (c)
			Result := rs.tuition = 750.0
		end

	test_polymorphism_feature_arguments
		local
			s1, s2, s3: STUDENT
			rs: RESIDENT_STUDENT
			nrs: NON_RESIDENT_STUDENT
			sms: STUDENT_MANAGEMENT_SYSTEM
		do
			create sms.make
			create {STUDENT} s1.make ("s1")
			create {RESIDENT_STUDENT} s2.make ("s2")
			create {NON_RESIDENT_STUDENT} s3.make ("s3")
			create {RESIDENT_STUDENT} rs.make ("rs")
			create {NON_RESIDENT_STUDENT} nrs.make ("nrs")

			sms.add_s (s1)
			sms.add_s (s2)
			sms.add_s (s3)
			sms.add_s (rs)
			sms.add_s (nrs)

--			sms.add_rs (s1)
--			sms.add_rs (s2)
--			sms.add_rs (s3)
			sms.add_rs (rs)
--			sms.add_rs (nrs)

--			sms.add_nrs (s1)
--			sms.add_nrs (s2)
--			sms.add_nrs (s3)
--			sms.add_nrs (rs)
			sms.add_nrs (nrs)
		end

	test_sms_polymorphism: BOOLEAN
		local
			rs: RESIDENT_STUDENT
			nrs: NON_RESIDENT_STUDENT
			c: COURSE
			sms: STUDENT_MANAGEMENT_SYSTEM
		do
			comment ("test_sms_polymorphism")
			create rs.make ("Jim")
			rs.set_pr (1.5)
			create nrs.make ("Jeremy")
			nrs.set_dr (0.5)

			create sms.make
			sms.add_s (rs)
			sms.add_s (nrs)
			create c.make ("EECS3311", 500)
			sms.register_all (c)

			Result :=
						sms.ss [1].tuition = 750
				and	sms.ss [2].tuition = 250
		end

	test_sms_dynamic_binding: BOOLEAN
		local
			rs: RESIDENT_STUDENT
			nrs: NON_RESIDENT_STUDENT
			c: COURSE
			sms: STUDENT_MANAGEMENT_SYSTEM
		do
			comment ("test_sms_dynamic_binding")
			create rs.make ("Jim")
			rs.set_pr (1.5)
			create nrs.make ("Jeremy")
			nrs.set_dr (0.5)

			create sms.make
			sms.add_s (rs)
			sms.add_s (nrs)
			create c.make ("EECS3311", 500)
			sms.register_all (c)

			Result :=
				sms.get_student (1).tuition = 750
				and
				sms.get_student (2).tuition = 250
		end

	test_smart_phone_type_cast
		local
			my_phone: IOS
		do
			create {IPHONE_6S_PLUS} my_phone.make
			-- feature inherited from SMART_PHONE
			my_phone.dial
			my_phone.surf_web
			-- feature introduced in IOS
			my_phone.facetime
			-- cannot call features specific to IPHONE_6S_PLUS
--			my_phone.three_d_touch
			-- cannot call features specific to ANDROID
--			my_phone.skype

			check attached {SMART_PHONE} my_phone as sp then
  				-- can now only call features defined in SMART_PHONE on sp
  				sp.dial
  				sp.surf_web
  				-- cannot call features introduced in IOS anymore
--				sp.facetime
				-- still cannot call features specific to IPHONE_6S_PLUS
--				sp.three_d_touch
				-- still cannot call features specific to ANDROID
--				sp.skype
			end

			check attached {IPHONE_6S_PLUS} my_phone as ip6s_plus then
  				-- can now call features defined in IPHONE_6S_PLUS on ip6s_plus
  				ip6s_plus.dial
  				ip6s_plus.surf_web
				ip6s_plus.facetime
				ip6s_plus.three_d_touch
				-- still cannot call features specific to ANDROID
--				sp.skype
			end
		end

	test_smart_phone_type_cast_violation
		local
			mine: ANDROID
		do
			create {SAMSUNG} mine.make
			-- ST of mine is ANDROID; DT of mine is SAMSUNG
			check attached {SMART_PHONE} mine as sp then
				-- ST of sp is SMART_PHONE; DT of sp is SAMSUNG
			end
			check attached {ANDROID} mine as android then
				-- ST of android is ANDROID; DT of android is SAMSUNG
			end
			check attached {SAMSUNG} mine as samsung then
				-- ST of samsung is SAMSNG; DT of samsung is SAMSUNG
			end
			check attached {HTC} mine as htc then
				-- Compiles because HTC is a descendant class of
				-- mine's ST (ANDROID)
				-- But assertion violation because HTC is not
				-- an ancestor class of mine's DT (SAMSUNG)
			end
		end
end
